package com.github.polpetta.mezzotre.util;

import static org.junit.jupiter.api.Assertions.*;

import com.google.common.util.concurrent.Service;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.parallel.Execution;
import org.junit.jupiter.api.parallel.ExecutionMode;

@Execution(ExecutionMode.CONCURRENT)
class ServiceModuleTest {

  /**
   * This test seems stupid, but it is actually important that the behavior of the extension is
   * always with a {@code lateInit} set to true, otherwise services won't be alive for the whole JVM
   * execution
   */
  @Test
  void shouldBeLateInit() {
    // Necessary otherwise ServiceManager will throw exception in the constructor 🤦
    final Service dumbService =
        new Service() {
          @Override
          public Service startAsync() {
            return null;
          }

          @Override
          public boolean isRunning() {
            return false;
          }

          @Override
          public State state() {
            return State.NEW;
          }

          @Override
          public Service stopAsync() {
            return null;
          }

          @Override
          public void awaitRunning() {}

          @Override
          public void awaitRunning(long timeout, TimeUnit unit) throws TimeoutException {}

          @Override
          public void awaitTerminated() {}

          @Override
          public void awaitTerminated(long timeout, TimeUnit unit) throws TimeoutException {}

          @Override
          public Throwable failureCause() {
            return null;
          }

          @Override
          public void addListener(Listener listener, Executor executor) {}
        };

    final ServiceModule serviceModule = new ServiceModule(List.of(dumbService));

    assertTrue(serviceModule.lateinit());
  }
}
